package test;

import javax.annotation.Resource;
import javax.ejb.CreateException;
import javax.ejb.EJB;
import javax.ejb.Handle;
import javax.ejb.SessionContext;
import javax.ejb.Stateless;
import javax.jws.WebService;
import javax.jws.WebMethod;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.rmi.RemoteException;
import java.util.logging.Level;
import java.util.logging.Logger;

@Stateless @WebService
@EJB(name = "test.Sful", mappedName = "MySfulBean", beanInterface = SfulHome.class)
public class TestService
{
    private static final Logger log = Logger.getLogger(TestService.class.getName());

    @Resource private SessionContext ctx;

    public byte[] doThis()
    {
        try
        {
            SfulHome svcHome = (SfulHome)ctx.lookup("test.Sful");
            Sful svc = svcHome.create();
            svc.setString("this is a test");
            return encodeHandle(svc.getHandle());
        }
        catch (CreateException e) { log.log(Level.SEVERE, "Couldn't create SfulBean", e); }
        catch (RemoteException e) { log.log(Level.SEVERE, "Remote exception with SfulBean", e); }
        return null;
    }


    public boolean doThat(byte[] sessionID)
    {
        Sful svc = decodeHandle(sessionID);
        try
        {
            String test = svc.getString();
            return test.equals("this is a test");
        }
        catch (RemoteException e) { log.log(Level.SEVERE, "Problem executing", e); }
        return false;
    }

    @WebMethod(exclude = true)
    private byte[] encodeHandle(Handle handle)
    {
        try
        {
            ByteArrayOutputStream fo = new ByteArrayOutputStream();
            ObjectOutputStream so = new ObjectOutputStream(fo);
            so.writeObject(handle);
            so.flush();
            so.close();
            return fo.toByteArray();
        }
        catch (IOException e) { log.log(Level.WARNING, "Error converting session handle to String", e); }
        return null;
    }

    @WebMethod(exclude = true)
    private Sful decodeHandle(byte[] handle)
    {
        try
        {
            InputStream io = new ByteArrayInputStream(handle);
            ObjectInputStream os = new ObjectInputStream(io);
            Handle sessionHandle = (Handle)os.readObject();
            return (Sful)sessionHandle.getEJBObject();
        }
        catch (ClassNotFoundException e) { log.log(Level.WARNING, "Bad session handle", e); }
        catch (IOException e)            { log.log(Level.WARNING, "IO error reading session handle", e); }
        return null;
    }
}
